@@ -1,7 +1,7 @@
 /**************************************************************************
  *   global.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2019 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2019 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -33,11 +33,33 @@ volatile sig_atomic_t the_window_resized = FALSE;
 
 bool on_a_vt = FALSE;
 		/* Whether we're running on a Linux console (a VT). */
+bool shifted_metas = FALSE;
+		/* Whether any Sh-M-<letter> combo has been bound. */
 
 bool meta_key;
 		/* Whether the current keystroke is a Meta key. */
 bool shift_held;
 		/* Whether Shift was being held together with a movement key. */
+bool mute_modifiers = FALSE;
+		/* Whether to ignore modifier keys while running a macro or string bind. */
+bool bracketed_paste = FALSE;
+		/* Whether text is being pasted into nano from outside. */
+
+bool started_curses = FALSE;
+		/* Becomes TRUE when curses mode has been entered the first time. */
+bool we_are_running = FALSE;
+		/* Becomes TRUE as soon as all options and files have been read. */
+bool more_than_one = FALSE;
+		/* Whether more than one buffer is or has been open. */
+
+bool inhelp = FALSE;
+		/* Whether we are in the help viewer. */
+char *title = NULL;
+		/* When not NULL: the title of the current help text. */
+
+bool refresh_needed = FALSE;
+		/* Did a command mangle enough of the buffer that we should
+		 * repaint the screen? */
 bool focusing = TRUE;
 		/* Whether an update of the edit window should center the cursor. */
 
@@ -47,8 +69,6 @@ bool as_an_at = TRUE;
 bool control_C_was_pressed = FALSE;
 		/* Whether Ctrl+C was pressed (when a keyboard interrupt is enabled). */
 
-bool started_curses = FALSE;
-
 bool suppress_cursorpos = FALSE;
 		/* Should we skip constant position display for current keystroke? */
 
@@ -58,22 +78,24 @@ message_type lastmessage = HUSH;
 linestruct *pletion_line = NULL;
 		/* The line where the last completion was found, if any. */
 
-bool inhelp = FALSE;
-		/* Whether we are in the help viewer. */
-char *title = NULL;
-		/* When not NULL: the title of the current help text. */
-
-bool more_than_one = FALSE;
-		/* Whether more than one buffer is or has been open. */
 bool also_the_last = FALSE;
 		/* Whether indenting/commenting should include the last line of
 		 * the marked region. */
-bool is_shorter;
-		/* Whether a row's text is narrower than the screen's width. */
 
+char *answer = NULL;
+		/* The answer string used by the status-bar prompt. */
+
+char *last_search = NULL;
+		/* The last string we searched for. */
 int didfind = 0;
 		/* Whether the last search found something. */
 
+char *present_path = NULL;
+		/* The current browser directory when trying to do tab completion. */
+
+unsigned flags[4] = {0, 0, 0, 0};
+		/* Our flags array, containing the states of all global options. */
+
 int controlleft, controlright, controlup, controldown;
 int controlhome, controlend;
 #ifndef NANO_TINY
@@ -93,15 +115,6 @@ size_t wrap_at = 0;
 		/* The actual column where we will wrap lines, based on fill. */
 #endif
 
-char *last_search = NULL;
-		/* The last string we searched for. */
-
-char *present_path = NULL;
-		/* The current browser directory when trying to do tab completion. */
-
-unsigned flags[4] = {0, 0, 0, 0};
-		/* Our flag containing the states of all global options. */
-
 WINDOW *topwin = NULL;
 		/* The top portion of the screen, showing the version number of nano,
 		 * the name of the file, and whether the buffer was modified. */
@@ -110,7 +123,7 @@ WINDOW *edit = NULL;
 		 * contents of the current buffer, the file we are editing. */
 WINDOW *bottomwin = NULL;
 		/* The bottom portion of the screen, where we display statusbar
-		 * messages, the statusbar prompt, and a list of shortcuts. */
+		 * messages, the status-bar prompt, and a list of shortcuts. */
 int editwinrows = 0;
 		/* How many rows does the edit window take up? */
 int editwincols = -1;
@@ -160,20 +173,12 @@ regex_t quotereg;
 char *word_chars = NULL;
 		/* Nonalphanumeric characters that also form words. */
 
-char *answer = NULL;
-		/* The answer string used by the statusbar prompt. */
-
 ssize_t tabsize = -1;
 		/* The width of a tab in spaces.  The default is set in main(). */
 
 #ifndef NANO_TINY
 char *backup_dir = NULL;
 		/* The directory where we store backup files. */
-
-const char *locking_prefix = ".";
-		/* Prefix of how to store the vim-style lock file. */
-const char *locking_suffix = ".swp";
-		/* Suffix of the vim-style lock file. */
 #endif
 #ifdef ENABLE_OPERATINGDIR
 char *operating_dir = NULL;
@@ -194,10 +199,6 @@ bool have_palette = FALSE;
 		/* Whether the colors for the current syntax have been initialized. */
 #endif
 
-bool refresh_needed = FALSE;
-		/* Did a command mangle enough of the buffer that we should
-		 * repaint the screen? */
-
 int currmenu = MMOST;
 		/* The currently active menu, initialized to a dummy value. */
 keystruct *sclist = NULL;
@@ -248,24 +249,13 @@ char *homedir = NULL;
 char *statedir = NULL;
 		/* The directory for nano's history files. */
 
-#ifdef ENABLE_NANORC
-#define NUMBER_OF_MENUS  16
-char *menunames[NUMBER_OF_MENUS] = { "main", "search", "replace", "replacewith",
-									"yesno", "gotoline", "writeout", "insert",
-									"extcmd", "help", "spell", "linter",
-									"browser", "whereisfile", "gotodir",
-									"all" };
-int menusymbols[NUMBER_OF_MENUS] = { MMAIN, MWHEREIS, MREPLACE, MREPLACEWITH,
-									MYESNO, MGOTOLINE, MWRITEFILE, MINSERTFILE,
-									MEXTCMD, MHELP, MSPELL, MLINTER,
-									MBROWSER, MWHEREISFILE, MGOTODIR,
-									MMOST|MBROWSER|MHELP|MYESNO };
-#endif
-
 #if defined(ENABLE_NANORC) || defined(ENABLE_HISTORIES)
 char *startup_problem = NULL;
 		/* An error message (if any) about nanorc files or history files. */
 #endif
+#ifdef ENABLE_NANORC
+char *custom_nanorc = NULL;
+#endif
 
 bool spotlighted = FALSE;
 	/* Whether any text is spotlighted. */
@@ -302,14 +292,17 @@ void flip_goto(void)
 {
 }
 #ifdef ENABLE_BROWSER
-void to_files_void(void)
+void to_files(void)
 {
 }
-void goto_dir_void(void)
+void goto_dir(void)
 {
 }
 #endif
 #ifndef NANO_TINY
+void do_nothing(void)
+{
+}
 void do_toggle_void(void)
 {
 }
@@ -381,38 +374,37 @@ void add_to_sclist(int menus, const char *scstring, const int keycode,
 #ifndef NANO_TINY
 	static int counter = 0;
 #endif
-	keystruct *s = nmalloc(sizeof(keystruct));
+	keystruct *sc = nmalloc(sizeof(keystruct));
 
 	/* Start the list, or tack on the next item. */
 	if (sclist == NULL)
-		sclist = s;
+		sclist = sc;
 	else
-		tailsc->next = s;
-	s->next = NULL;
+		tailsc->next = sc;
+	sc->next = NULL;
 
 	/* Fill in the data. */
-	s->menus = menus;
-	s->func = func;
+	sc->menus = menus;
+	sc->func = func;
 #ifndef NANO_TINY
-	s->toggle = toggle;
+	sc->toggle = toggle;
 	/* When not the same toggle as the previous one, increment the ID. */
 	if (toggle)
-		s->ordinal = (tailsc->toggle == toggle) ? counter : ++counter;
+		sc->ordinal = (tailsc->toggle == toggle) ? counter : ++counter;
 #endif
-	assign_keyinfo(s, scstring, keycode);
+	sc->keystr = scstring;
+	sc->keycode = (keycode ? keycode : keycode_from_string(scstring));
 
-	tailsc = s;
+	tailsc = sc;
 }
 
 /* Return the first shortcut in the list of shortcuts that
  * matches the given func in the given menu. */
 const keystruct *first_sc_for(int menu, void (*func)(void))
 {
-	const keystruct *s;
-
-	for (s = sclist; s != NULL; s = s->next)
-		if ((s->menus & menu) && s->func == func)
-			return s;
+	for (keystruct *sc = sclist; sc != NULL; sc = sc->next)
+		if ((sc->menus & menu) && sc->func == func)
+			return sc;
 
 	return NULL;
 }
@@ -421,13 +413,14 @@ const keystruct *first_sc_for(int menu, void (*func)(void))
  * current menu, if any; otherwise, return the given default value. */
 int the_code_for(void (*func)(void), int defaultval)
 {
-	const keystruct *s = first_sc_for(currmenu, func);
+	const keystruct *sc = first_sc_for(currmenu, func);
 
-	if (s == NULL)
+	if (sc == NULL)
 		return defaultval;
 
-	meta_key = s->meta;
-	return s->keycode;
+	meta_key = (0x20 <= sc->keycode && sc->keycode <= 0x7E);
+
+	return sc->keycode;
 }
 
 /* Return the number of entries that can be shown in the given menu. */
@@ -450,86 +443,125 @@ size_t shown_entries_for(int menu)
 	return count;
 }
 
-/* Return the shortcut that corresponds to the values of kbinput (the
- * key itself) and meta_key (whether the key is a meta sequence).  The
- * returned shortcut will be the first in the list that corresponds to
- * the given sequence. */
-const keystruct *get_shortcut(int *kbinput)
+/* Return the first shortcut in the current menu that matches the given input. */
+const keystruct *get_shortcut(int *keycode)
 {
-	/* Plain characters cannot be shortcuts, so just skip those. */
-	if (!meta_key && ((*kbinput >= 0x20 && *kbinput < 0x7F) ||
-						(*kbinput >= 0xA0 && *kbinput <= 0xFF)))
+	/* Plain characters and upper control codes cannot be shortcuts. */
+	if (!meta_key && 0x20 <= *keycode && *keycode <= 0xFF)
 		return NULL;
 
-	for (keystruct *s = sclist; s != NULL; s = s->next) {
-		if ((s->menus & currmenu) && *kbinput == s->keycode &&
-										meta_key == s->meta)
-			return s;
+	/* Lower control codes with Meta cannot be shortcuts either. */
+	if (meta_key && *keycode < 0x20)
+		return NULL;
+
+	/* During a paste at a prompt, ignore all command keycodes. */
+	if (bracketed_paste && *keycode != BRACKETED_PASTE_MARKER)
+		return NULL;
+
+	for (keystruct *sc = sclist; sc != NULL; sc = sc->next) {
+		if ((sc->menus & currmenu) && *keycode == sc->keycode)
+			return sc;
 	}
 
 	return NULL;
 }
 
 /* Return a pointer to the function that is bound to the given key. */
-functionptrtype func_from_key(int *kbinput)
+functionptrtype func_from_key(int *keycode)
 {
-	const keystruct *s = get_shortcut(kbinput);
+	const keystruct *sc = get_shortcut(keycode);
 
-	if (s)
-		return s->func;
-	else
-		return NULL;
+	return (sc) ? sc->func : NULL;
 }
 
+#if defined(ENABLE_BROWSER) || defined(ENABLE_HELP)
+/* Return the function that is bound to the given key in the file browser or
+ * the help viewer.  Accept also certain plain characters, for compatibility
+ * with Pico or to mimic 'less' and similar text viewers. */
+functionptrtype interpret(int *keycode)
+{
+	if (!meta_key) {
+		if (*keycode == 'N')
+			return do_findprevious;
+		if (*keycode == 'n')
+			return do_findnext;
+
+		switch (tolower(*keycode)) {
+			case '-':
+				return do_page_up;
+			case ' ':
+				return do_page_down;
+			case 'w':
+			case '/':
+				return do_search_forward;
+#ifdef ENABLE_BROWSER
+			case 'g':
+				return goto_dir;
+#endif
+			case '?':
+				return do_help;
+			case 's':
+				return do_enter;
+			case 'e':
+			case 'q':
+			case 'x':
+				return do_exit;
+		}
+	}
+
+	return func_from_key(keycode);
+}
+#endif /* ENABLE_BROWSER || ENABLE_HELP */
+
 /* Parse the given keystring and return the corresponding keycode,
  * or return -1 when the string is invalid. */
 int keycode_from_string(const char *keystring)
 {
 	if (keystring[0] == '^') {
-		if (strcasecmp(keystring, "^Space") == 0)
-			return 0;
+		if (keystring[2] == '\0') {
+			if (keystring[1] == '/')
+				return 31;
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
-		if (strcasecmp(keystring, "^H") == 0)
-			return KEY_BACKSPACE;
+			if (keystring[1] == 'H')
+				return KEY_BACKSPACE;
 #endif
-		if (keystring[1] == '/' && strlen(keystring) == 2)
-			return 31;
-		if (keystring[1] <= '_' && strlen(keystring) == 2)
-			return keystring[1] - 64;
+			if (keystring[1] <= '_')
+				return keystring[1] - 64;
+			else if (keystring[1] == '`')
+				return 0;
+			else
+				return -1;
+		} else if (strcasecmp(keystring, "^Space") == 0)
+			return 0;
 		else
 			return -1;
 	} else if (keystring[0] == 'M') {
+		if (keystring[1] == '-' && keystring[3] == '\0')
+			return tolower((unsigned char)keystring[2]);
 		if (strcasecmp(keystring, "M-Space") == 0)
 			return (int)' ';
-		if (keystring[1] == '-' && strlen(keystring) == 3)
-			return tolower((unsigned char)keystring[2]);
 		else
 			return -1;
+#ifdef ENABLE_NANORC
+	} else if (strncasecmp(keystring, "Sh-M-", 5) == 0 &&
+				'a' <= (keystring[5] | 0x20) && (keystring[5] | 0x20) <= 'z' &&
+				keystring[6] == '\0') {
+		shifted_metas = TRUE;
+		return (keystring[5] & 0x5F);
+#endif
 	} else if (keystring[0] == 'F') {
 		int fn = atoi(&keystring[1]);
 		if (fn < 1 || fn > 24)
 			return -1;
 		return KEY_F0 + fn;
-	} else if (!strcasecmp(keystring, "Ins"))
+	} else if (strcasecmp(keystring, "Ins") == 0)
 		return KEY_IC;
-	else if (!strcasecmp(keystring, "Del"))
+	else if (strcasecmp(keystring, "Del") == 0)
 		return KEY_DC;
 	else
 		return -1;
 }
 
-/* Set the string and its corresponding keycode for the given shortcut s. */
-void assign_keyinfo(keystruct *s, const char *keystring, const int keycode)
-{
-	s->keystr = keystring;
-	s->meta = (keystring[0] == 'M' && keycode == 0);
-
-	if (keycode)
-		s->keycode = keycode;
-	else
-		s->keycode = keycode_from_string(keystring);
-}
-
 /* These two tags are used elsewhere too, so they are global. */
 /* TRANSLATORS: Try to keep the next two strings at most 10 characters. */
 const char *exit_tag = N_("Exit");
@@ -543,8 +575,7 @@ void shortcut_init(void)
 	 * they are best kept shorter than 56 characters, but may be longer. */
 	const char *cancel_gist = N_("Cancel the current function");
 	const char *help_gist = N_("Display this help text");
-	const char *exit_gist =
-		N_("Close the current buffer / Exit from nano");
+	const char *exit_gist = N_("Close the current buffer / Exit from nano");
 	const char *writeout_gist =
 		N_("Write the current buffer (or the marked region) to disk");
 	const char *readfile_gist =
@@ -553,19 +584,6 @@ void shortcut_init(void)
 		N_("Search forward for a string or a regular expression");
 	const char *wherewas_gist =
 		N_("Search backward for a string or a regular expression");
-#ifdef ENABLE_BROWSER
-	const char *browserwhereis_gist = N_("Search forward for a string");
-	const char *browserwherewas_gist = N_("Search backward for a string");
-	const char *browserrefresh_gist = N_("Refresh the file list");
-#ifndef NANO_TINY
-	const char *browserlefthand_gist = N_("Go to lefthand column");
-	const char *browserrighthand_gist = N_("Go to righthand column");
-	const char *browsertoprow_gist = N_("Go to first row in this column");
-	const char *browserbottomrow_gist = N_("Go to last row in this column");
-#endif
-#endif
-	const char *prevpage_gist = N_("Go one screenful up");
-	const char *nextpage_gist = N_("Go one screenful down");
 	const char *cut_gist =
 		N_("Cut current line (or marked region) and store it in cutbuffer");
 	const char *uncut_gist =
@@ -602,6 +620,8 @@ void shortcut_init(void)
 	const char *paraend_gist =
 		N_("Go just beyond end of paragraph; then of next paragraph");
 #endif
+	const char *prevpage_gist = N_("Go one screenful up");
+	const char *nextpage_gist = N_("Go one screenful down");
 	const char *firstline_gist = N_("Go to the first line of the file");
 	const char *lastline_gist = N_("Go to the last line of the file");
 #ifndef NANO_TINY
@@ -657,12 +677,9 @@ void shortcut_init(void)
 	const char *recordmacro_gist = N_("Start/stop recording a macro");
 	const char *runmacro_gist = N_("Run the last recorded macro");
 #endif
-	const char *case_gist =
-		N_("Toggle the case sensitivity of the search");
-	const char *reverse_gist =
-		N_("Reverse the direction of the search");
-	const char *regexp_gist =
-		N_("Toggle the use of regular expressions");
+	const char *case_gist = N_("Toggle the case sensitivity of the search");
+	const char *reverse_gist = N_("Reverse the direction of the search");
+	const char *regexp_gist = N_("Toggle the use of regular expressions");
 #ifdef ENABLE_HISTORIES
 	const char *older_gist =
 		N_("Recall the previous search/replace string");
@@ -691,6 +708,15 @@ void shortcut_init(void)
 	const char *lastfile_gist = N_("Go to the last file in the list");
 	const char *backfile_gist = N_("Go to the previous file in the list");
 	const char *forwardfile_gist = N_("Go to the next file in the list");
+#ifndef NANO_TINY
+	const char *browserlefthand_gist = N_("Go to lefthand column");
+	const char *browserrighthand_gist = N_("Go to righthand column");
+	const char *browsertoprow_gist = N_("Go to first row in this column");
+	const char *browserbottomrow_gist = N_("Go to last row in this column");
+#endif
+	const char *browserwhereis_gist = N_("Search forward for a string");
+	const char *browserwherewas_gist = N_("Search backward for a string");
+	const char *browserrefresh_gist = N_("Refresh the file list");
 	const char *gotodir_gist = N_("Go to directory");
 #endif
 #ifdef ENABLE_COLOR
@@ -712,7 +738,7 @@ void shortcut_init(void)
 
 	/* Start populating the different menus with functions. */
 
-	add_to_funcs(do_help_void, (MMOST | MBROWSER) & ~MFINDINHELP,
+	add_to_funcs(do_help, (MMOST | MBROWSER) & ~MFINDINHELP,
 		/* TRANSLATORS: Try to keep the next eleven strings at most 10 characters. */
 		N_("Get Help"), WITHORSANS(help_gist), TOGETHER, VIEW);
 
@@ -831,7 +857,7 @@ void shortcut_init(void)
 #endif
 
 #ifdef ENABLE_BROWSER
-	add_to_funcs(goto_dir_void, MBROWSER,
+	add_to_funcs(goto_dir, MBROWSER,
 		/* TRANSLATORS: Try to keep the next seven strings at most 10 characters. */
 		N_("Go To Dir"), WITHORSANS(gotodir_gist), TOGETHER, VIEW);
 
@@ -852,16 +878,16 @@ void shortcut_init(void)
 		/* TRANSLATORS: This starts a backward search. */
 		N_("Where Was"), WITHORSANS(wherewas_gist), TOGETHER, VIEW);
 #endif
-	add_to_funcs(do_findprevious, MMAIN|MHELP|MBROWSER,
+	add_to_funcs(do_findprevious, MMAIN|MBROWSER|MHELP,
 		/* TRANSLATORS: This refers to searching the preceding occurrence. */
 		N_("Previous"), WITHORSANS(findprev_gist), TOGETHER, VIEW);
 #ifndef NANO_TINY
-	add_to_funcs(do_findnext, MMAIN|MHELP|MBROWSER,
+	add_to_funcs(do_findnext, MMAIN|MBROWSER|MHELP,
 		N_("Next"), WITHORSANS(findnext_gist), BLANKAFTER, VIEW);
 #else
 	/* In the tiny version, arrange things a bit differently, so that
 	 * the useful M-W and M-Q are shown in the help lines. */
-	add_to_funcs(do_findnext, MMAIN|MHELP|MBROWSER,
+	add_to_funcs(do_findnext, MMAIN|MBROWSER|MHELP,
 		N_("Next"), WITHORSANS(findnext_gist), TOGETHER, VIEW);
 	add_to_funcs(do_search_backward, MMAIN|MHELP,
 		N_("Where Was"), WITHORSANS(wherewas_gist), BLANKAFTER, VIEW);
@@ -892,9 +918,9 @@ void shortcut_init(void)
 	add_to_funcs(do_end, MMAIN,
 		N_("End"), WITHORSANS(end_gist), BLANKAFTER, VIEW);
 
-	add_to_funcs(do_up, MMAIN|MHELP|MBROWSER,
+	add_to_funcs(do_up, MMAIN|MBROWSER|MHELP,
 		N_("Prev Line"), WITHORSANS(prevline_gist), TOGETHER, VIEW);
-	add_to_funcs(do_down, MMAIN|MHELP|MBROWSER,
+	add_to_funcs(do_down, MMAIN|MBROWSER|MHELP,
 		N_("Next Line"), WITHORSANS(nextline_gist), TOGETHER, VIEW);
 #if !defined(NANO_TINY) || defined(ENABLE_HELP)
 	add_to_funcs(do_scroll_up, MMAIN,
@@ -1070,7 +1096,7 @@ void shortcut_init(void)
 #ifdef ENABLE_BROWSER
 	/* The file browser is only available when not in restricted mode. */
 	if (!ISSET(RESTRICTED))
-		add_to_funcs(to_files_void, MWRITEFILE|MINSERTFILE,
+		add_to_funcs(to_files, MWRITEFILE|MINSERTFILE,
 			N_("To Files"), WITHORSANS(tofiles_gist), TOGETHER, VIEW);
 
 	add_to_funcs(do_page_up, MBROWSER,
@@ -1112,19 +1138,19 @@ void shortcut_init(void)
 	add_to_sclist(MMOST, "Bsp", KEY_BACKSPACE, do_backspace, 0);
 	add_to_sclist(MMOST, "Sh-Del", SHIFT_DELETE, do_backspace, 0);
 	add_to_sclist(MMOST, "^D", 0, do_delete, 0);
-	add_to_sclist(MMOST, "Del", 0, do_delete, 0);
+	add_to_sclist(MMOST, "Del", KEY_DC, do_delete, 0);
 	add_to_sclist(MMOST, "^I", 0, do_tab, 0);
 	add_to_sclist(MMOST, "Tab", TAB_CODE, do_tab, 0);
-	add_to_sclist((MMOST|MBROWSER) & ~MFINDINHELP, "^G", 0, do_help_void, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "^X", 0, do_exit, 0);
+	add_to_sclist((MMOST|MBROWSER) & ~MFINDINHELP, "^G", 0, do_help, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "^X", 0, do_exit, 0);
 	if (!ISSET(PRESERVE))
 		add_to_sclist(MMAIN, "^S", 0, do_savefile, 0);
 	add_to_sclist(MMAIN, "^O", 0, do_writeout_void, 0);
 	add_to_sclist(MMAIN, "^R", 0, do_insertfile_void, 0);
-	add_to_sclist(MMAIN, "Ins", 0, do_insertfile_void, 0);
+	add_to_sclist(MMAIN, "Ins", KEY_IC, do_insertfile_void, 0);
 	if (!ISSET(PRESERVE))
-		add_to_sclist(MMAIN|MHELP|MBROWSER, "^Q", 0, do_search_backward, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "^W", 0, do_search_forward, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "^Q", 0, do_search_backward, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "^W", 0, do_search_forward, 0);
 	add_to_sclist(MMAIN, "^\\", 0, do_replace, 0);
 	add_to_sclist(MMAIN, "M-R", 0, do_replace, 0);
 	add_to_sclist(MMOST, "^K", 0, cut_text, 0);
@@ -1144,19 +1170,19 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN, "^C", 0, do_cursorpos_void, 0);
 	add_to_sclist(MMAIN, "^_", 0, do_gotolinecolumn_void, 0);
 	add_to_sclist(MMAIN, "M-G", 0, do_gotolinecolumn_void, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "^Y", 0, do_page_up, 0);
-	add_to_sclist(MHELP|MBROWSER, "Bsp", KEY_BACKSPACE, do_page_up, 0);
-	add_to_sclist(MHELP|MBROWSER, "Sh-Del", SHIFT_DELETE, do_page_up, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "PgUp", KEY_PPAGE, do_page_up, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "^V", 0, do_page_down, 0);
-	add_to_sclist(MHELP|MBROWSER, "Space", 0x20, do_page_down, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "PgDn", KEY_NPAGE, do_page_down, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "^Y", 0, do_page_up, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "PgUp", KEY_PPAGE, do_page_up, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "^V", 0, do_page_down, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "PgDn", KEY_NPAGE, do_page_down, 0);
+	add_to_sclist(MBROWSER|MHELP, "Bsp", KEY_BACKSPACE, do_page_up, 0);
+	add_to_sclist(MBROWSER|MHELP, "Sh-Del", SHIFT_DELETE, do_page_up, 0);
+	add_to_sclist(MBROWSER|MHELP, "Space", 0x20, do_page_down, 0);
 	add_to_sclist(MMAIN|MHELP, "M-\\", 0, to_first_line, 0);
 	add_to_sclist(MMAIN|MHELP, "^Home", CONTROL_HOME, to_first_line, 0);
 	add_to_sclist(MMAIN|MHELP, "M-/", 0, to_last_line, 0);
 	add_to_sclist(MMAIN|MHELP, "^End", CONTROL_END, to_last_line, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "M-W", 0, do_findnext, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "M-Q", 0, do_findprevious, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "M-W", 0, do_findnext, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "M-Q", 0, do_findprevious, 0);
 #ifndef NANO_TINY
 	add_to_sclist(MMAIN, "M-]", 0, do_find_bracket, 0);
 	add_to_sclist(MMAIN, "M-A", 0, do_mark, 0);
@@ -1186,8 +1212,8 @@ void shortcut_init(void)
 	add_to_sclist(MMOST|MBROWSER, "^F", 0, do_right, 0);
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
-		add_to_sclist(MMOST|MHELP|MBROWSER, "\xE2\x97\x80", KEY_LEFT, do_left, 0);
-		add_to_sclist(MMOST|MHELP|MBROWSER, "\xE2\x96\xb6", KEY_RIGHT, do_right, 0);
+		add_to_sclist(MMOST|MBROWSER|MHELP, "\xE2\x97\x80", KEY_LEFT, do_left, 0);
+		add_to_sclist(MMOST|MBROWSER|MHELP, "\xE2\x96\xb6", KEY_RIGHT, do_right, 0);
 		add_to_sclist(MSOME, "^\xE2\x97\x80", CONTROL_LEFT, do_prev_word_void, 0);
 		add_to_sclist(MSOME, "^\xE2\x96\xb6", CONTROL_RIGHT, do_next_word_void, 0);
 #if !defined(NANO_TINY) && defined(ENABLE_MULTIBUFFER)
@@ -1199,8 +1225,8 @@ void shortcut_init(void)
 	} else
 #endif
 	{
-		add_to_sclist(MMOST|MHELP|MBROWSER, "Left", KEY_LEFT, do_left, 0);
-		add_to_sclist(MMOST|MHELP|MBROWSER, "Right", KEY_RIGHT, do_right, 0);
+		add_to_sclist(MMOST|MBROWSER|MHELP, "Left", KEY_LEFT, do_left, 0);
+		add_to_sclist(MMOST|MBROWSER|MHELP, "Right", KEY_RIGHT, do_right, 0);
 		add_to_sclist(MSOME, "^Left", CONTROL_LEFT, do_prev_word_void, 0);
 		add_to_sclist(MSOME, "^Right", CONTROL_RIGHT, do_next_word_void, 0);
 #ifdef ENABLE_MULTIBUFFER
@@ -1216,25 +1242,25 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN, "M-F", 0, do_next_word_void, 0);
 	add_to_sclist(MMAIN, "M-N", 0, do_next_word_void, 0);
 #endif
-	add_to_sclist(MMOST|MBROWSER, "M-Space", 0, do_prev_word_void, 0);
-	add_to_sclist(MMOST|MBROWSER, "^Space", 0, do_next_word_void, 0);
+	add_to_sclist(MMOST, "M-Space", 0, do_prev_word_void, 0);
+	add_to_sclist(MMOST, "^Space", 0, do_next_word_void, 0);
 	add_to_sclist(MMOST, "^A", 0, do_home, 0);
 	add_to_sclist(MMOST, "Home", KEY_HOME, do_home, 0);
 	add_to_sclist(MMOST, "^E", 0, do_end, 0);
 	add_to_sclist(MMOST, "End", KEY_END, do_end, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "^P", 0, do_up, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "^N", 0, do_down, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "^P", 0, do_up, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "^N", 0, do_down, 0);
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
-		add_to_sclist(MMAIN|MHELP|MBROWSER, "\xE2\x96\xb2", KEY_UP, do_up, 0);
-		add_to_sclist(MMAIN|MHELP|MBROWSER, "\xE2\x96\xbc", KEY_DOWN, do_down, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "\xE2\x96\xb2", KEY_UP, do_up, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "\xE2\x96\xbc", KEY_DOWN, do_down, 0);
 		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xb2", CONTROL_UP, do_prev_block, 0);
 		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xbc", CONTROL_DOWN, do_next_block, 0);
 	} else
 #endif
 	{
-		add_to_sclist(MMAIN|MHELP|MBROWSER, "Up", KEY_UP, do_up, 0);
-		add_to_sclist(MMAIN|MHELP|MBROWSER, "Down", KEY_DOWN, do_down, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "Up", KEY_UP, do_up, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "Down", KEY_DOWN, do_down, 0);
 		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^Up", CONTROL_UP, do_prev_block, 0);
 		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^Down", CONTROL_DOWN, do_next_block, 0);
 	}
@@ -1276,10 +1302,11 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN, "M-D", 0, do_wordlinechar_count, 0);
 #endif
 #ifdef ENABLE_JUSTIFY
-	add_to_sclist(MMAIN|MWHEREIS, "M-J", 0, do_full_justify, 0);
+	if (!ISSET(VIEW_MODE))
+		add_to_sclist(MMAIN|MWHEREIS, "M-J", 0, do_full_justify, 0);
 #endif
 	if (!ISSET(PRESERVE))
-		add_to_sclist(MMAIN|MHELP|MBROWSER, "^L", 0, total_refresh, 0);
+		add_to_sclist(MMAIN|MBROWSER|MHELP, "^L", 0, total_refresh, 0);
 	else
 		add_to_sclist(MMAIN|MBROWSER, "^L", 0, total_refresh, 0);
 	add_to_sclist(MMAIN, "^Z", 0, do_suspend_void, 0);
@@ -1354,8 +1381,8 @@ void shortcut_init(void)
 	add_to_sclist(MBROWSER, "End", KEY_END, to_last_file, 0);
 	add_to_sclist(MBROWSER, "^Home", CONTROL_HOME, to_first_file, 0);
 	add_to_sclist(MBROWSER, "^End", CONTROL_END, to_last_file, 0);
-	add_to_sclist(MBROWSER, "^_", 0, goto_dir_void, 0);
-	add_to_sclist(MBROWSER, "M-G", 0, goto_dir_void, 0);
+	add_to_sclist(MBROWSER, "^_", 0, goto_dir, 0);
+	add_to_sclist(MBROWSER, "M-G", 0, goto_dir, 0);
 #endif
 	if (ISSET(TEMP_FILE) && !ISSET(PRESERVE))
 		add_to_sclist(MWRITEFILE, "^Q", 0, discard_buffer, 0);
@@ -1384,9 +1411,9 @@ void shortcut_init(void)
 #ifdef ENABLE_BROWSER
 	/* Only when not in restricted mode, allow entering the file browser. */
 	if (!ISSET(RESTRICTED))
-		add_to_sclist(MWRITEFILE|MINSERTFILE, "^T", 0, to_files_void, 0);
+		add_to_sclist(MWRITEFILE|MINSERTFILE, "^T", 0, to_files, 0);
 #endif
-	add_to_sclist(MHELP|MBROWSER, "^C", 0, do_exit, 0);
+	add_to_sclist(MBROWSER|MHELP, "^C", 0, do_exit, 0);
 	/* Allow exiting from the file browser and the help viewer with
 	 * the same key as they were entered. */
 #ifdef ENABLE_BROWSER
@@ -1394,28 +1421,32 @@ void shortcut_init(void)
 #endif
 #ifdef ENABLE_HELP
 	add_to_sclist(MHELP, "^G", 0, do_exit, 0);
-	add_to_sclist(MHELP, "F1", 0, do_exit, 0);
+	add_to_sclist(MHELP, "F1", KEY_F(1), do_exit, 0);
 	add_to_sclist(MHELP, "Home", KEY_HOME, to_first_line, 0);
 	add_to_sclist(MHELP, "End", KEY_END, to_last_line, 0);
 #endif
 #ifdef ENABLE_COLOR
 	add_to_sclist(MLINTER, "^X", 0, do_cancel, 0);
 #endif
-	add_to_sclist(MMOST & ~MFINDINHELP, "F1", 0, do_help_void, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "F2", 0, do_exit, 0);
-	add_to_sclist(MMAIN, "F3", 0, do_writeout_void, 0);
+	add_to_sclist(MMOST & ~MFINDINHELP, "F1", KEY_F(1), do_help, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "F2", KEY_F(2), do_exit, 0);
+	add_to_sclist(MMAIN, "F3", KEY_F(3), do_writeout_void, 0);
 #ifdef ENABLE_JUSTIFY
-	add_to_sclist(MMAIN, "F4", 0, do_justify_void, 0);
+	add_to_sclist(MMAIN, "F4", KEY_F(4), do_justify_void, 0);
 #endif
-	add_to_sclist(MMAIN, "F5", 0, do_insertfile_void, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "F6", 0, do_search_forward, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "F7", 0, do_page_up, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "F8", 0, do_page_down, 0);
-	add_to_sclist(MMOST, "F9", 0, cut_text, 0);
-	add_to_sclist(MMOST, "F10", 0, paste_text, 0);
-	add_to_sclist(MMAIN, "F11", 0, do_cursorpos_void, 0);
+	add_to_sclist(MMAIN, "F5", KEY_F(5), do_insertfile_void, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP, "F6", KEY_F(6), do_search_forward, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "F7", KEY_F(7), do_page_up, 0);
+	add_to_sclist(MMAIN|MBROWSER|MHELP|MLINTER, "F8", KEY_F(8), do_page_down, 0);
+	add_to_sclist(MMOST, "F9", KEY_F(9), cut_text, 0);
+	add_to_sclist(MMOST, "F10", KEY_F(10), paste_text, 0);
+	add_to_sclist(MMAIN, "F11", KEY_F(11), do_cursorpos_void, 0);
 #ifdef ENABLE_SPELLER
-	add_to_sclist(MMAIN, "F12", 0, do_spell, 0);
+	add_to_sclist(MMAIN, "F12", KEY_F(12), do_spell, 0);
+#endif
+#ifndef NANO_TINY
+	/* Catch and ignore bracketed paste marker keys. */
+	add_to_sclist(MMOST|MBROWSER|MHELP|MYESNO, "", BRACKETED_PASTE_MARKER, do_nothing, 0);
 #endif
 }
 
@@ -1457,295 +1488,3 @@ const char *flagtostr(int flag)
 	}
 }
 #endif /* !NANO_TINY */
-
-#ifdef ENABLE_NANORC
-/* Interpret a function string given in the rc file, and return a
- * shortcut record with the corresponding function filled in. */
-keystruct *strtosc(const char *input)
-{
-	keystruct *s = nmalloc(sizeof(keystruct));
-
-#ifndef NANO_TINY
-	s->toggle = 0;
-#endif
-
-#ifdef ENABLE_HELP
-	if (!strcasecmp(input, "help"))
-		s->func = do_help_void;
-	else
-#endif
-	if (!strcasecmp(input, "cancel"))
-		s->func = do_cancel;
-	else if (!strcasecmp(input, "exit"))
-		s->func = do_exit;
-	else if (!strcasecmp(input, "discardbuffer"))
-		s->func = discard_buffer;
-	else if (!strcasecmp(input, "writeout"))
-		s->func = do_writeout_void;
-	else if (!strcasecmp(input, "savefile"))
-		s->func = do_savefile;
-	else if (!strcasecmp(input, "insert"))
-		s->func = do_insertfile_void;
-	else if (!strcasecmp(input, "whereis"))
-		s->func = do_search_forward;
-	else if (!strcasecmp(input, "wherewas"))
-		s->func = do_search_backward;
-	else if (!strcasecmp(input, "findprevious"))
-		s->func = do_findprevious;
-	else if (!strcasecmp(input, "findnext"))
-		s->func = do_findnext;
-	else if (!strcasecmp(input, "replace"))
-		s->func = do_replace;
-	else if (!strcasecmp(input, "cut"))
-		s->func = cut_text;
-	else if (!strcasecmp(input, "paste"))
-		s->func = paste_text;
-#ifndef NANO_TINY
-	else if (!strcasecmp(input, "cutrestoffile"))
-		s->func = cut_till_eof;
-	else if (!strcasecmp(input, "copy"))
-		s->func = copy_text;
-	else if (!strcasecmp(input, "zap"))
-		s->func = zap_text;
-	else if (!strcasecmp(input, "mark"))
-		s->func = do_mark;
-#endif
-#ifdef ENABLE_SPELLER
-	else if (!strcasecmp(input, "tospell") ||
-			 !strcasecmp(input, "speller"))
-		s->func = do_spell;
-#endif
-#ifdef ENABLE_COLOR
-	else if (!strcasecmp(input, "linter"))
-		s->func = do_linter;
-#ifdef ENABLE_SPELLER
-	else if (!strcasecmp(input, "formatter"))
-		s->func = do_formatter;
-#endif
-#endif
-	else if (!strcasecmp(input, "curpos"))
-		s->func = do_cursorpos_void;
-	else if (!strcasecmp(input, "gotoline"))
-		s->func = do_gotolinecolumn_void;
-#ifdef ENABLE_JUSTIFY
-	else if (!strcasecmp(input, "justify"))
-		s->func = do_justify_void;
-	else if (!strcasecmp(input, "fulljustify"))
-		s->func = do_full_justify;
-	else if (!strcasecmp(input, "beginpara"))
-		s->func = do_para_begin_void;
-	else if (!strcasecmp(input, "endpara"))
-		s->func = do_para_end_void;
-#endif
-#ifdef ENABLE_COMMENT
-	else if (!strcasecmp(input, "comment"))
-		s->func = do_comment;
-#endif
-#ifdef ENABLE_WORDCOMPLETION
-	else if (!strcasecmp(input, "complete"))
-		s->func = complete_a_word;
-#endif
-#ifndef NANO_TINY
-	else if (!strcasecmp(input, "indent"))
-		s->func = do_indent;
-	else if (!strcasecmp(input, "unindent"))
-		s->func = do_unindent;
-	else if (!strcasecmp(input, "chopwordleft") ||
-			 !strcasecmp(input, "cutwordleft"))  /* Deprecated; remove in 2021. */
-		s->func = chop_previous_word;
-	else if (!strcasecmp(input, "chopwordright") ||
-			 !strcasecmp(input, "cutwordright"))  /* Deprecated; remove in 2021. */
-		s->func = chop_next_word;
-	else if (!strcasecmp(input, "findbracket"))
-		s->func = do_find_bracket;
-	else if (!strcasecmp(input, "wordcount"))
-		s->func = do_wordlinechar_count;
-	else if (!strcasecmp(input, "recordmacro"))
-		s->func = record_macro;
-	else if (!strcasecmp(input, "runmacro"))
-		s->func = run_macro;
-	else if (!strcasecmp(input, "undo"))
-		s->func = do_undo;
-	else if (!strcasecmp(input, "redo"))
-		s->func = do_redo;
-#endif
-	else if (!strcasecmp(input, "left") ||
-			 !strcasecmp(input, "back"))
-		s->func = do_left;
-	else if (!strcasecmp(input, "right") ||
-			 !strcasecmp(input, "forward"))
-		s->func = do_right;
-	else if (!strcasecmp(input, "up") ||
-			 !strcasecmp(input, "prevline"))
-		s->func = do_up;
-	else if (!strcasecmp(input, "down") ||
-			 !strcasecmp(input, "nextline"))
-		s->func = do_down;
-#if !defined(NANO_TINY) || defined(ENABLE_HELP)
-	else if (!strcasecmp(input, "scrollup"))
-		s->func = do_scroll_up;
-	else if (!strcasecmp(input, "scrolldown"))
-		s->func = do_scroll_down;
-#endif
-	else if (!strcasecmp(input, "prevword"))
-		s->func = do_prev_word_void;
-	else if (!strcasecmp(input, "nextword"))
-		s->func = do_next_word_void;
-	else if (!strcasecmp(input, "home"))
-		s->func = do_home;
-	else if (!strcasecmp(input, "end"))
-		s->func = do_end;
-	else if (!strcasecmp(input, "prevblock"))
-		s->func = do_prev_block;
-	else if (!strcasecmp(input, "nextblock"))
-		s->func = do_next_block;
-	else if (!strcasecmp(input, "pageup") ||
-			 !strcasecmp(input, "prevpage"))
-		s->func = do_page_up;
-	else if (!strcasecmp(input, "pagedown") ||
-			 !strcasecmp(input, "nextpage"))
-		s->func = do_page_down;
-	else if (!strcasecmp(input, "firstline"))
-		s->func = to_first_line;
-	else if (!strcasecmp(input, "lastline"))
-		s->func = to_last_line;
-#ifdef ENABLE_MULTIBUFFER
-	else if (!strcasecmp(input, "prevbuf"))
-		s->func = switch_to_prev_buffer;
-	else if (!strcasecmp(input, "nextbuf"))
-		s->func = switch_to_next_buffer;
-#endif
-	else if (!strcasecmp(input, "verbatim"))
-		s->func = do_verbatim_input;
-	else if (!strcasecmp(input, "tab"))
-		s->func = do_tab;
-	else if (!strcasecmp(input, "enter"))
-		s->func = do_enter;
-	else if (!strcasecmp(input, "delete"))
-		s->func = do_delete;
-	else if (!strcasecmp(input, "backspace"))
-		s->func = do_backspace;
-	else if (!strcasecmp(input, "refresh"))
-		s->func = total_refresh;
-	else if (!strcasecmp(input, "suspend"))
-		s->func = do_suspend_void;
-	else if (!strcasecmp(input, "casesens"))
-		s->func = case_sens_void;
-	else if (!strcasecmp(input, "regexp"))
-		s->func = regexp_void;
-	else if (!strcasecmp(input, "backwards"))
-		s->func = backwards_void;
-	else if (!strcasecmp(input, "flipreplace"))
-		s->func = flip_replace;
-	else if (!strcasecmp(input, "flipgoto"))
-		s->func = flip_goto;
-#ifdef ENABLE_HISTORIES
-	else if (!strcasecmp(input, "older"))
-		s->func = get_history_older_void;
-	else if (!strcasecmp(input, "newer"))
-		s->func = get_history_newer_void;
-#endif
-#ifndef NANO_TINY
-	else if (!strcasecmp(input, "dosformat"))
-		s->func = dos_format_void;
-	else if (!strcasecmp(input, "macformat"))
-		s->func = mac_format_void;
-	else if (!strcasecmp(input, "append"))
-		s->func = append_void;
-	else if (!strcasecmp(input, "prepend"))
-		s->func = prepend_void;
-	else if (!strcasecmp(input, "backup"))
-		s->func = backup_file_void;
-	else if (!strcasecmp(input, "flipexecute"))
-		s->func = flip_execute;
-	else if (!strcasecmp(input, "flippipe"))
-		s->func = flip_pipe;
-	else if (!strcasecmp(input, "flipconvert"))
-		s->func = flip_convert;
-#endif
-#ifdef ENABLE_MULTIBUFFER
-	else if (!strcasecmp(input, "flipnewbuffer"))
-		s->func = flip_newbuffer;
-#endif
-#ifdef ENABLE_BROWSER
-	else if (!strcasecmp(input, "tofiles") ||
-			 !strcasecmp(input, "browser"))
-		s->func = to_files_void;
-	else if (!strcasecmp(input, "gotodir"))
-		s->func = goto_dir_void;
-	else if (!strcasecmp(input, "firstfile"))
-		s->func = to_first_file;
-	else if (!strcasecmp(input, "lastfile"))
-		s->func = to_last_file;
-#endif
-	else {
-#ifndef NANO_TINY
-		s->func = do_toggle_void;
-		if (!strcasecmp(input, "nohelp"))
-			s->toggle = NO_HELP;
-		else if (!strcasecmp(input, "constantshow"))
-			s->toggle = CONSTANT_SHOW;
-		else if (!strcasecmp(input, "softwrap"))
-			s->toggle = SOFTWRAP;
-#ifdef ENABLE_LINENUMBERS
-		else if (!strcasecmp(input, "linenumbers"))
-			s->toggle = LINE_NUMBERS;
-#endif
-		else if (!strcasecmp(input, "whitespacedisplay"))
-			s->toggle = WHITESPACE_DISPLAY;
-#ifdef ENABLE_COLOR
-		else if (!strcasecmp(input, "nosyntax"))
-			s->toggle = NO_COLOR_SYNTAX;
-#endif
-		else if (!strcasecmp(input, "smarthome"))
-			s->toggle = SMART_HOME;
-		else if (!strcasecmp(input, "autoindent"))
-			s->toggle = AUTOINDENT;
-		else if (!strcasecmp(input, "cutfromcursor"))
-			s->toggle = CUT_FROM_CURSOR;
-#ifdef ENABLE_WRAPPING
-		else if (!strcasecmp(input, "nowrap"))
-			s->toggle = BREAK_LONG_LINES;
-#endif
-		else if (!strcasecmp(input, "tabstospaces"))
-			s->toggle = TABS_TO_SPACES;
-#ifdef ENABLE_MOUSE
-		else if (!strcasecmp(input, "mouse"))
-			s->toggle = USE_MOUSE;
-#endif
-		else if (!strcasecmp(input, "suspendenable"))
-			s->toggle = SUSPEND;
-		else
-#endif /* !NANO_TINY */
-		{
-			free(s);
-			return NULL;
-		}
-	}
-	return s;
-}
-
-/* Return the symbol that corresponds to the given menu name. */
-int name_to_menu(const char *name)
-{
-	int index = -1;
-
-	while (++index < NUMBER_OF_MENUS)
-		if (strcasecmp(name, menunames[index]) == 0)
-			return menusymbols[index];
-
-	return -1;
-}
-
-/* Return the name that corresponds to the given menu symbol. */
-char *menu_to_name(int menu)
-{
-	int index = -1;
-
-	while (++index < NUMBER_OF_MENUS)
-		if (menusymbols[index] == menu)
-			return menunames[index];
-
-	return "boooo";
-}
-#endif /* ENABLE_NANORC */
